# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
# flake8: noqa

from stripe.api_resources.identity.verification_report import (
    VerificationReport as VerificationReport,
)
from stripe.api_resources.identity.verification_session import (
    VerificationSession as VerificationSession,
)
