# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
# flake8: noqa

from stripe.api_resources.billing_portal.configuration import (
    Configuration as Configuration,
)
from stripe.api_resources.billing_portal.session import Session as Session
